@@ -62,49 +62,55 @@
 #' @param returnintens  Should the image of the estimated intensity be returned? Defaults to \code{TRUE}.
 #' @export
 #' @return An object of class "\code{highriskzone}", which is a list of
-#'    \item{ typehrz, criterion, cutoff }{ see arguments}
+#'    \item{ typehrz, criterion, cutoff, nxprob}{ see arguments}
 #'    \item{ zone }{ Determined high-risk zone: Object of class "owin" based on a binary mask.
 #'                   See \code{\link[spatstat]{owin}}. }
-#'    \item{ threshold }{ determined threshold. If criterion="area", it is either the distance (if type="dist")
-#' or the threshold c (for type="intens"). If criterion="indirect", it is either the quantile of the
-#' nearest-neighbour distance which is used as radius (if type="dist") or the threshold c (for type="intens"). If criterion="direct",
-#' it equals the cutoff for both types.}
+#'    \item{ threshold }{ determined threshold. If type = "dist" and criterion = "direct" it is the specified radius.
+#'    If criterion = "indirect" or "area" the determined radius used to construct a risk zone fulfilling the specified criterion 
+#'    and cutoff. If type = "dist" it is the specified or calculated threshold c, the maximum intensitiy of unexploded bombs 
+#'    outside the risk zone.}
 #'    \item{ calccutoff }{ determined cutoff-value. For type="dist" and criterion="area", this is the
-#' quantile of the nearest-neighbour distance. For type="intens" and criterion="area", it is the failure
+#' quantile of the nearest-neighbour distance. For type="intens" and criterion="area" or "direct", it is the failure
 #' probability alpha. For all other criterions it is NA.}
 #'    \item{ covmatrix }{ If not given (and \code{type="intens"}), it is estimated. See \code{\link[ks]{Hscv}}.}
 #'    \item{ estint }{ Estimated intensity. See \code{\link[spatstat]{density.ppp}}.}
 #' @seealso \code{\link[spatstat]{distmap}}, \code{\link[spatstat]{eval.im}}, \code{\link[spatstat]{owin}}
 #' @examples
-#' \dontrun{
-#'  data(craterA)
-#'  spatstat::spatstat.options(npixel=400)
+#' set.seed(1211515)
+#' data(craterA)
+#' #change npixel = 100 to 1000 to get a nicer picture
+#' spatstat::spatstat.options(npixel=100)
+#' # reduce number of observations for faster computation
+#' craterA <- craterA[sample(1:craterA$n, 150)]
+#' # define restriction area
+#' restrwin <- spatstat::owin(xrange=craterA$window$xrange, yrange=craterA$window$yrange,
+#'                            poly=list(x=c(1500, 1500, 2000, 2000), y=c(2000, 1500, 1500, 2000)))
 #'
-#'# define restriction area
-#'restrwin <- owin(xrange=craterA$window$xrange, yrange=craterA$window$yrange,
-#'  poly=list(x=c(1500, 1500, 2000, 2000), y=c(2000, 1500, 1500, 2000)))
-#'
-#'# create image of observation probability (30% inside restriction area)
-#'wim <- as.im(craterA$window, value=1)
-#'rim <- as.im(restrwin, xy=list(x=wim$xcol, y=wim$yrow))
-#'rim$v[is.na(rim$v)] <- 0
-#'oim1 <- eval.im(wim - 0.7 * rim)
-#'
-#'# determine high-risk zone by weighting the observations
-#'hrzi1 <- det_hrz_restr(ppdata=craterA, type = "intens", criterion = "indirect",
-#'  cutoff = 0.4, hole=NULL, obsprobs=NULL, obsprobimage=oim1, nxprob = 0.1)
-#'
-#'# determine high-risk zone by accounting for a hole
-#'hrzi2 <- det_hrz_restr(ppdata=craterA, type = "intens", criterion = "indirect",
-#'  cutoff = 0.4, hole=restrwin, obsprobs=NULL, obsprobimage=NULL, nxprob = 0.1)
-#'}
+#' # create image of observation probability (30% inside restriction area)
+#' wim <- spatstat::as.im(craterA$window, value=1)
+#' rim <- spatstat::as.im(restrwin, xy=list(x=wim$xcol, y=wim$yrow))
+#' rim$v[is.na(rim$v)] <- 0
+#' oim1 <- spatstat::eval.im(wim - 0.7 * rim)
+#' # determine high-risk zone by weighting the observations
+#' hrzi1 <- det_hrz_restr(ppdata=craterA, type = "intens", criterion = "indirect",
+#'                        cutoff = 0.4, hole=NULL, obsprobs=NULL, obsprobimage=oim1, nxprob = 0.1)
+#' 
+#' # determine high-risk zone by accounting for a hole
+#' hrzi2 <- det_hrz_restr(ppdata=craterA, type = "intens", criterion = "indirect",
+#'                        cutoff = 0.4, hole=restrwin, obsprobs=NULL, obsprobimage=NULL, nxprob = 0.1)
+ 
 
 det_hrz_restr <- function(ppdata, type, criterion, cutoff, hole=NULL, integratehole=TRUE, 
                           obsprobs=NULL, obsprobimage=NULL, distancemap=NULL, intens=NULL, 
                           nxprob=0.1, covmatrix=NULL, returnintens=TRUE){
   
   win <- ppdata$window
-  calccutoff <- NA    
+  calccutoff <- NA
+  type <- match.arg(type, choices=c("intens", "dist"))  
+  criterion <- match.arg(criterion, choices=c("area", "indirect", "direct"))
+  check_det_hrz_restr_input(ppdata = ppdata, type = type, criterion = criterion, cutoff = cutoff,
+                            hole=hole, integratehole=integratehole, obsprobs=obsprobs, obsprobimage=obsprobimage,
+                      distancemap = distancemap, intens = intens, nxprob = nxprob, covmatrix = covmatrix)
   
   if (!is.null(hole)){
     winminus <- setminus.owin(A=ppdata$window, B=hole)
@@ -158,7 +164,7 @@ det_hrz_restr <- function(ppdata, type, criterion, cutoff, hole=NULL, integrateh
     
     if(is.null(intens)){
       
-      estim <- est_intens(ppdata, covmatrix=covmatrix, weights=1/invweight)
+      estim <- est_intens_weight(ppdata, covmatrix=covmatrix, weights=1/invweight)
       intens <- estim$intensest
       covmatrix <- estim$covmatrix
       
@@ -166,18 +172,25 @@ det_hrz_restr <- function(ppdata, type, criterion, cutoff, hole=NULL, integrateh
     
     if(criterion=="area"){
       
-      res_det_thresholdfromarea <- det_thresholdfromarea(win=win, intens=intens, nxprob=nxprob, areahrz=cutoff)
+      res_det_thresholdfromarea <- det_thresholdfromarea_rest(win=win, intens=intens, nxprob=nxprob, 
+                                                              areahrz=cutoff, hole = hole, 
+                                                              integratehole = integratehole)
       threshold <- res_det_thresholdfromarea$thresh
       calccutoff <- res_det_thresholdfromarea$calccutoff
       
     }else{
       
-      threshold <- ifelse(criterion=="indirect", det_threshold(intens=intens, alpha=cutoff, nxprob=nxprob), cutoff)
+      threshold <- ifelse(criterion=="indirect", det_threshold(intens=intens, alpha=cutoff, nxprob=nxprob), 
+                          (1-nxprob)/nxprob * cutoff)
       
     }
     
     HRZimage <- eval.im(intens > threshold)
     
+    if(criterion == "direct")
+      calccutoff <- det_alpha(intens, threshold = threshold, nxprob = nxprob)
+    
+    threshold <- nxprob / (1-nxprob) * threshold
   }
   
   if (is.null(hole) | !integratehole){
@@ -190,11 +203,56 @@ det_hrz_restr <- function(ppdata, type, criterion, cutoff, hole=NULL, integrateh
 
   Rwindow <- owin(xrange=win$xrange, yrange=win$yrange, mask=as.matrix(totalim))
   if(returnintens){
-    result <- list(typehrz=type, criterion=criterion, cutoff=cutoff, zone=Rwindow, threshold=threshold, calccutoff=calccutoff, covmatrix=covmatrix, estint=intens)
+    result <- list(typehrz=type, criterion=criterion, cutoff=cutoff, nxprob = nxprob, zone=Rwindow, threshold=threshold, calccutoff=calccutoff, covmatrix=covmatrix, estint=intens)
   }else{
-    result <- list(typehrz=type, criterion=criterion, cutoff=cutoff, zone=Rwindow, threshold=threshold, calccutoff=calccutoff, covmatrix=covmatrix)
+    result <- list(typehrz=type, criterion=criterion, cutoff=cutoff, nxprob = nxprob, zone=Rwindow, threshold=threshold, calccutoff=calccutoff, covmatrix=covmatrix)
   }
   class(result) <- "highriskzone"
   return(result)
   
-}
\ No newline at end of file
+}
+
+#' Checks the arguments of det_hrz_restr
+#' 
+#' For each argument it is checked if it is of a correct value or class.
+#' @inheritParams det_hrz_restr
+#' 
+check_det_hrz_restr_input <- function(ppdata, type, criterion, cutoff, hole, integratehole, 
+                                      obsprobs, obsprobimage, distancemap, intens, 
+                                      nxprob, covmatrix, returnintens){
+  
+  #errors
+  #check if arguments have correct values
+  if ( !is.ppp(ppdata) ) {
+    stop("data is not of class ppp")
+  }
+  
+  #this is for every criterion
+  stopifnot( cutoff > 0 | length(cutoff) != 1 ) 
+  #special: alpha and the quantile can only be in [0,1]
+  if ( criterion == "indirect" ) stopifnot( cutoff < 1 ) 
+  
+  if ( !is.im(distancemap) & !is.null(distancemap) ) stop("distancemap must be of class im (see distmap) or NULL.")
+  
+  if(!is.null(hole) & !is.owin(hole)) stop("hole must be of class owin")
+  stopifnot(is.logical(integratehole))
+  if(!is.null(obsprobs))
+    stopifnot(length(obsprobs) == ppdata$n | any(obsprobs>1 | obsprobs<0))
+  
+  if(!is.null(obsprobimage)) {
+    stopifnot(is.im(obsprobimage))
+    if(!is.null(obsprobs))
+      warning("obsprobs and obsprobimage specified, obsprobs are used")
+  }
+  
+  if ( !is.im(intens) & !is.null(intens) ) stop("wrong input for intens. The intensity must be of class im (see density.ppp) or NULL.")
+  
+  if ( !(nxprob > 0 & nxprob < 1) ) stop("nxprob is a probability and therefore it has to be in the interval [0, 1]")
+  
+  if ( !is.null(covmatrix) && (!isSymmetric(covmatrix) | !all(eigen(covmatrix)$values > 0)) ){
+    stop("covmatrix has to be symmetric and positive semidefinit")
+  
+  stopifnot(is.logical(returnintens))  
+  }
+  
+}
